<?php
/**
 * Plugin Name: Disable All Update & Notification
 * Description: Manage your WordPress environments effortlessly with the Disable All Update & Notification plugin. Disable core, plugin, and theme updates, and eliminate update notifications for a streamlined experience.
 * Version: 1.0.7
 * Author: Pirtpal Singh
 * License: GNU General Public License v2 or later
 * License URI: http://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: disable-all-update-notification
 *
 * @package disable-all-update-notification
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	// Ensure that the script is terminated to prevent direct access.
	die;
}



class PSDWN_Disable_All_Update_Notification { 
	/**
	 * Constructor to initialize the plugin.
	 */
	public function __construct() {
		// Initialize the admin settings.
		add_action( 'admin_init', array( $this, 'psdwn_admin_init' ) );

		// Disable Theme Updates.
		add_filter( 'pre_transient_update_themes', array( $this, 'psdwn_last_checked_atm' ) );
		add_filter( 'pre_site_transient_update_themes', array( $this, 'psdwn_last_checked_atm' ) );

		// Disable Plugin Updates.
		add_action( 'pre_transient_update_plugins', array( $this, 'psdwn_last_checked_atm' ) );

		// Disable Core Updates.
		add_filter( 'pre_transient_update_core', array( $this, 'psdwn_last_checked_atm' ) );
		add_filter( 'pre_site_transient_update_core', array( $this, 'psdwn_last_checked_atm' ) );

		// Filter schedule checks.
		add_action( 'schedule_event', array( $this, 'psdwn_filter_cron_events' ) );
		add_action( 'pre_set_site_transient_update_themes', array( $this, 'psdwn_last_checked_atm' ), 21, 1 );

		// Disable All Automatic Updates 3.7+.
		$this->psdwn_disable_all_automatic_updates_v3_7();

		// Define constants for settings if not already defined.
		$this->psdwn_define_constants();

		// Block outgoing requests.
		add_filter( 'pre_http_request', array( $this, 'psdwn_block_request' ), 10, 3 );
	}

	/**
	 * Helper method to disable All Automatic Updates 3.7+.
	 */
	private function psdwn_disable_all_automatic_updates_v3_7() {
		$filter_functions = array(
			'auto_update_core',
			'auto_update_theme',
			'auto_update_plugin',
			'wp_auto_update_core',
			'auto_update_translation',
			'automatic_updater_disabled',
			'auto_core_update_send_email',
			'allow_dev_auto_core_updates',
			'allow_major_auto_core_updates',
			'allow_minor_auto_core_updates',
			'automatic_updates_is_vcs_checkout',
			'automatic_updates_send_debug_email',
			'send_core_update_notification_email',
		);

		foreach ( $filter_functions as $filter_function ) {
			add_filter( $filter_function, '__return_false' );
		}
	}

	/**
	 * Helper method to define constants for settings if not already defined.
	 */
	private function psdwn_define_constants() {
		$constants = array(
			'AUTOMATIC_UPDATER_DISABLED' => true,
			'WP_AUTO_UPDATE_CORE' => false,
			'DISALLOW_FILE_MODS' => false,
			'DISALLOW_FILE_EDIT' => false,
		);

		foreach ( $constants as $constant => $value ) {
			if (!defined($constant)) {
				define($constant,$value);
			}
		}
	}


	/**
	 * Initialize and load the plugin settings.
	 */
	public function psdwn_admin_init() {
		// Check if the necessary functions are available.
		if ( ! function_exists( 'remove_action' ) ) {
			return;
		}

		// Disable 'update plugins' option from bulk operations select list.
		global $current_user;
		$current_user->allcaps['update_plugins'] = 0;

		// Hide maintenance and update nag messages.
		$this->psdwn_remove_nags();

		// Disable Theme Updates.
		$this->psdwn_disable_theme_updates();

		// Disable Plugin Updates.
		$this->psdwn_disable_plugin_updates();

		// Disable Core Updates.
		$this->psdwn_disable_core_updates();

		// Disable All Automatic Updates 3.7+.
		$this->psdwn_disable_all_automatic_updates();
	}

	/**
	 * Helper method to remove maintenance and update nag messages.
	 */
	private function psdwn_remove_nags() {
		remove_action( 'admin_notices', 'update_nag', 3 );
		remove_action( 'network_admin_notices', 'update_nag', 3 );
		remove_action( 'admin_notices', 'maintenance_nag' );
		remove_action( 'network_admin_notices', 'maintenance_nag' );
	}

	/**
	 * Helper method to disable Theme Updates.
	 */
	private function psdwn_disable_theme_updates() {
		remove_action( 'load-themes.php', 'wp_update_themes' );
		remove_action( 'load-update.php', 'wp_update_themes' );
		remove_action( 'admin_init', '_maybe_update_themes' );
		remove_action( 'wp_update_themes', 'wp_update_themes' );
		wp_clear_scheduled_hook( 'wp_update_themes' );

		// For WordPress 3.0+
		remove_action( 'load-update-core.php', 'wp_update_themes' );
		wp_clear_scheduled_hook( 'wp_update_themes' );
	}

	/**
	 * Helper method to disable Plugin Updates.
	 */
	private function psdwn_disable_plugin_updates() {
		remove_action( 'load-plugins.php', 'wp_update_plugins' );
		remove_action( 'load-update.php', 'wp_update_plugins' );
		remove_action( 'admin_init', '_maybe_update_plugins' );
		remove_action( 'wp_update_plugins', 'wp_update_plugins' );
		wp_clear_scheduled_hook( 'wp_update_plugins' );

		// For WordPress 3.0+
		remove_action( 'load-update-core.php', 'wp_update_plugins' );
		wp_clear_scheduled_hook( 'wp_update_plugins' );
	}

	/**
	 * Helper method to disable Core Updates.
	 */
	private function psdwn_disable_core_updates() {
		add_filter( 'pre_option_update_core', '__return_null' );

		remove_action( 'wp_version_check', 'wp_version_check' );
		remove_action( 'admin_init', '_maybe_update_core' );
		wp_clear_scheduled_hook( 'wp_version_check' );

		// For WordPress 3.0+
		wp_clear_scheduled_hook( 'wp_version_check' );
	}

	/**
	 * Helper method to disable All Automatic Updates 3.7+.
	 */
	private function psdwn_disable_all_automatic_updates() {
		remove_action( 'wp_maybe_auto_update', 'wp_maybe_auto_update' );
		remove_action( 'admin_init', 'wp_maybe_auto_update' );
		remove_action( 'admin_init', 'wp_auto_update_core' );
		wp_clear_scheduled_hook( 'wp_maybe_auto_update' );
	}


	/**
	 * Check and block outgoing requests to specific URLs.
	 *
	 * @param bool   $pre  Whether to preempt the request. Default is false.
	 * @param array  $args HTTP request arguments.
	 * @param string $url  The URL of the request.
	 * @return bool|mixed Modified preempt value or original value.
	 */
	public function psdwn_block_request( $pre, $args, $url ) {
    // Check if the URL is empty.
    if ( empty( $url ) ) {
			return $pre;
    }

    // Parse the URL to get the host.
    $host = parse_url( $url, PHP_URL_HOST );

    // Check if the host is invalid.
    if ( ! $host ) {
			return $pre;
    }

    // Parse the URL again to get additional data.
    $url_data = parse_url( $url );

    // Check if the host is 'api.wordpress.org' and the path includes 'update-check' or 'browse-happy'.
    if ( false !== stripos( $host, 'api.wordpress.org' ) && ( false !== stripos( $url_data['path'], 'update-check' ) || false !== stripos( $url_data['path'], 'browse-happy' ) ) ) {
			// If the conditions are met, block the request.
			return true;
    }

    // Return the original value if no blocking is required.
    return $pre;
	}


	/**
	 * Filter cron events.
	 *
	 * @param stdClass $event The cron event object.
	 * @return stdClass|null Modified cron event object or null if filtered.
	 */
	public function psdwn_filter_cron_events( $event ) {
		// Check if the event hook matches certain conditions.
		$cron_events = [
			'wp_version_check',
			'wp_update_plugins',
			'wp_update_themes',
			'wp_maybe_auto_update' 
		];
		if ( in_array( $event->hook, $cron_events) ) {
			// If it matches, set the event to false to prevent execution.
			$event = null;
		}

		// Return the modified or null event object.
		return $event;
	}


	/**
	 * Override version check info.
	 *
	 * @param stdClass $transient The update transient object.
	 * @return stdClass Modified update transient object.
	 */
	public function psdwn_last_checked_atm( $transient ) {
		// Include WordPress version information.
		include( ABSPATH . WPINC . '/version.php' );

		// Create a new stdClass to store update information.
		$update_info = new stdClass;

		// Set updates property as an empty array.
		$update_info->updates = array();

		// Set version_checked to the current WordPress version.
		$update_info->version_checked = $wp_version;

		// Set last_checked to the current timestamp.
		$update_info->last_checked = time();

		// Return the modified update transient object.
		return $update_info;
	}

}

if (class_exists('PSDWN_Disable_All_Update_Notification')) {
  new PSDWN_Disable_All_Update_Notification();
}